#!/usr/local/bin/ruby -w
# by Kouichirou@Eto.com 2001/09/17

module SGL
  LINES = GL::LINES
  POINTS = GL::POINTS
  LINE_STRIP = GL::LINE_STRIP
  LINE_LOOP = GL::LINE_LOOP
  TRIANGLES = GL::TRIANGLES
  TRIANGLE_STRIP = GL::TRIANGLE_STRIP
  TRIANGLE_FAN = GL::TRIANGLE_FAN
  QUADS = GL::QUADS
  QUAD_STRIP = GL::QUAD_STRIP
  POLYGON = GL::POLYGON
  def beginObj(mode) GL.Begin(mode) end #----------------------------------------------------------------------ڍו`֌W
  def endObj() GL.End() end
  def vertex(a,b=nil,c=nil,d=nil)
    if a.is_a?(Array)
      GL.Vertex(a)
    elsif c==nil && d==nil
      GL.Vertex(a,b)
    elsif d==nil
      GL.Vertex(a,b,c)
    else
      GL.Vertex(a,b,c,d)
    end
  end
  def normal(a,b=nil,c=nil)
    if a.is_a?(Array)
      GL.Normal(a)
    else d==nil
      GL.Normal(a,b,c)
    end
  end
  module_function :beginObj, :endObj, :vertex, :normal

  def translate(a,b,c) GL.Translate(a,b,c) end #----------------------------------------------------------------------s񑀍֌W
  def rotateX(a) GL.Rotate(a,1,0,0) end
  def rotateY(a) GL.Rotate(a,0,1,0) end
  def rotateZ(a) GL.Rotate(a,0,0,1) end
  def scale(a) GL.Scale(a,a,a) end
  module_function :translate, :rotateX, :rotateY, :rotateZ, :scale
  
  def point(a,b,c=nil) #----------------------------------------------------------------------Zk`֌W
    GL.Begin(GL::POINTS)
    if c==nil #2D
      GL.Vertex(a,b)
    else
      GL.Vertex(a,b,c)
    end
    GL.End()
  end
  def line(a,b,c,d,e=nil,f=nil)
    GL.Begin(GL::LINES) 
    if e==nil || f==nil #2D
      GL.Vertex(a,b)
      GL.Vertex(c,d)
    else
      GL.Vertex(a,b,c)
      GL.Vertex(d,e,f)
    end
    GL.End()
  end
  def rect(a,b,c,d,e=nil,f=nil)
    if e==nil || f==nil #2D
      GL.Rect(a,b,c,d)
    else
      GL.Rect(a,b,c,d,e,f)
    end
  end
  def triangle(a,b,c,d,e,f)
    GL.Begin(GL::TRIANGLES)
    GL.Vertex(a,b)
    GL.Vertex(c,d)
    GL.Vertex(e,f)
    GL.End()
  end
  def box(x1,y1,z1,x2,y2,z2)
    box =
      [[x2,y2, z1], 
      [x2, y1, z1],
      [x1, y1, z1],
      [x1, y2, z1],
      [x1, y2, z2],
      [x2, y2, z2],
      [x2, y1, z2],
      [x1, y1, z2]]
    GL.Begin(GL::QUADS)
    #GL.Color(1.0, 0.0, 0.0);
    GL.Vertex(box[0]);
    GL.Vertex(box[1]);
    GL.Vertex(box[2]);
    GL.Vertex(box[3]);
    #GL.Color(0.0, 1.0, 0.0);
    GL.Vertex(box[3]);
    GL.Vertex(box[4]);
    GL.Vertex(box[7]);
    GL.Vertex(box[2]);
    #GL.Color(0.0, 0.0, 1.0);
    GL.Vertex(box[0]);
    GL.Vertex(box[5]);
    GL.Vertex(box[6]);
    GL.Vertex(box[1]);
    #GL.Color(0.0, 1.0, 1.0);
    GL.Vertex(box[5]);
    GL.Vertex(box[4]);
    GL.Vertex(box[7]);
    GL.Vertex(box[6]);
    #GL.Color(1.0, 1.0, 0.0);
    GL.Vertex(box[5]);
    GL.Vertex(box[0]);
    GL.Vertex(box[3]);
    GL.Vertex(box[4]);
    #GL.Color(1.0, 0.0, 1.0);
    GL.Vertex(box[6]);
    GL.Vertex(box[1]);
    GL.Vertex(box[2]);
    GL.Vertex(box[7]);
    GL.End()
  end
  def cube(x,y,z,s)
    s = s/2
    box(x-s,y-s,z-s,x+s,y+s,z+s)
  end
  module_function :point, :line, :rect, :triangle, :box, :cube
end

module SGL_notuse
  class Obj3D #----------------------------------------------------------------------
    def initialize
      @x,@y,@z, @rx,@ry,@rz = 0,0,0, 0,0,0
    end
    attr_reader :x,:y,:z
    def rotateX(r) @rx += r; self; end
    def rotateY(r) @ry += r; self; end
    def rotateZ(r) @rz += r; self; end
    def move(x,y,z) @x+=x; @y+=y; @z+=z; self; end
    def moveTo(x,y,z) @x=x; @y=y; @z=z; self; end
    def draw() end
  end
  class Cube < Obj3D #----------------------------------------------------------------------
    def initialize(size)
      super()
      @size = size
    end
    def draw
      s = @size/2
      x1,y1,z1 = -s,-s,-s
      x2,y2,z2 = +s,+s,+s
      box =
	[[x2,y2, z1], 
	[x2, y1, z1],
	[x1, y1, z1],
	[x1, y2, z1],
	[x1, y2, z2],
	[x2, y2, z2],
	[x2, y1, z2],
	[x1, y1, z2]]
      GL.PushMatrix()
      GL.Translate(@x,@y,@z)
      GL.Rotate(@rx, 1,0,0)
      GL.Rotate(@ry, 0,1,0)
      GL.Rotate(@rz, 0,0,1)
      GL.Begin(GL::QUADS) 
      GL.Color(1.0, 0.0, 0.0);
      GL.Vertex(box[0]);
      GL.Vertex(box[1]);
      GL.Vertex(box[2]);
      GL.Vertex(box[3]);
      GL.Color(0.0, 1.0, 0.0);
      GL.Vertex(box[3]);
      GL.Vertex(box[4]);
      GL.Vertex(box[7]);
      GL.Vertex(box[2]);
      GL.Color(0.0, 0.0, 1.0);
      GL.Vertex(box[0]);
      GL.Vertex(box[5]);
      GL.Vertex(box[6]);
      GL.Vertex(box[1]);
      GL.Color(0.0, 1.0, 1.0);
      GL.Vertex(box[5]);
      GL.Vertex(box[4]);
      GL.Vertex(box[7]);
      GL.Vertex(box[6]);
      GL.Color(1.0, 1.0, 0.0);
      GL.Vertex(box[5]);
      GL.Vertex(box[0]);
      GL.Vertex(box[3]);
      GL.Vertex(box[4]);
      GL.Color(1.0, 0.0, 1.0);
      GL.Vertex(box[6]);
      GL.Vertex(box[1]);
      GL.Vertex(box[2]);
      GL.Vertex(box[7]);
      GL.End()
      GL.PopMatrix()
    end
  end
end

#----------------------------------------------------------------------I
